package br.com.model.utilities;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import lombok.Getter;

import org.apache.commons.mail.EmailAttachment;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.HtmlEmail;
import org.apache.commons.mail.MultiPartEmail;
  

/**
 * @category utilities
 * @version 1.0
 * @param <InternetAddress>*/
public class Email<InternetAddress> {  
	
	private String authUser;
	private String authPass;
	private String host;
	private String auth;
	private int port;
	private String to;
	private String from;
	private String copiaEmailTo;
	private String anexosTo;

	private MultiPartEmail email;
	private  EmailAttachment anexo;
	
	

  public String getAuthUser() {
    return authUser;
  }

  public void setAuthUser(String authUser) {
    this.authUser = authUser;
  }

  public String getAuthPass() {
    return authPass;
  }

  public void setAuthPass(String authPass) {
    this.authPass = authPass;
  }

  public String getHost() {
    return host;
  }

  public void setHost(String host) {
    this.host = host;
  }

  public String getAuth() {
    return auth;
  }

  public void setAuth(String auth) {
    this.auth = auth;
  }

  public int getPort() {
    return port;
  }

  public void setPort(int port) {
    this.port = port;
  }

  public String getTo() {
    return to;
  }

  public void setTo(String to) {
    this.to = to;
  }

  public String getFrom() {
    return from;
  }

  public void setFrom(String from) {
    this.from = from;
  }

  public String getCopiaEmailTo() {
    return copiaEmailTo;
  }

  public void setCopiaEmailTo(String copiaEmailTo) {
    this.copiaEmailTo = copiaEmailTo;
  }

  public MultiPartEmail getEmail() {
    return email;
  }

  public void setEmail(MultiPartEmail email) {
    this.email = email;
  }

  public EmailAttachment getAnexo() {
    return anexo;
  }

  public void setAnexo(EmailAttachment anexo) {
    this.anexo = anexo;
  }
  
  

  public String getAnexosTo() {
    return anexosTo;
  }

  public void setAnexosTo(String anexosTo) {
    this.anexosTo = anexosTo;
  }

  private void initConf(String path){
		ManagerProperties managerProperties = new ManagerProperties();  
		try {
			Properties prop = managerProperties.getProp(path);
			
			this.host  = prop.getProperty("prop.email.host"); 
			this.port = Integer.parseInt(prop.getProperty("prop.email.port")); 
			this.authUser = prop.getProperty("prop.email.authuser");
			this.authPass = prop.getProperty("prop.email.authpass");
			this.auth = prop.getProperty("prop.email.auth");
			this.to = prop.getProperty("prop.email.to");
			this.from = prop.getProperty("prop.email.from");
			this.copiaEmailTo = prop.getProperty("prop.email.copiaEmailTo");
			this.anexosTo = prop.getProperty("prop.email.anexosTo");
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	/**
	 * @param String host, int port, String authUser, String authPass*/
	public Email(String host, int port, String authUser, String authPass, String auth, String to){
		this.host = host;
    	this.port = port;
    	this.authUser = authUser;
    	this.authPass = authPass;
    	this.auth = auth;
    	this.to = to;
    	this.email = new HtmlEmail();
    	
    	
    }  
	    
	/**
	 * @param String path. Caminho do arquivo mail.properties*/
    public Email(String path) {
    	this.initConf(path);
    	this.email = new MultiPartEmail();
    }
    
    /** 
     * @param  String nomeDestinatario, String emailDestinatario, String nomeRemetente, String emailRemetente, String assunto, String mensagem
     * @throws EmailException  
     * @throws MalformedURLException
     * @return String*/  
	@SuppressWarnings("unchecked")
  public String sendEmail( String nomeDestinatario, String emailDestinatario, String nomeRemetente, String emailRemetente, String assunto, String mensagem, boolean isHtml, List<InternetAddress> copiaEmailTo) throws EmailException {  
       
    	this.email.setHostName(this.host); // o servidor SMTP para envio do e-mail  
    	this.email.addTo(emailDestinatario, nomeDestinatario); //destinat�rio  
    	this.email.setFrom(emailRemetente, emailRemetente+", "); // remetente
    	this.email.setSubject(assunto); // assunto do e-mail
    	this.email.setCc((Collection<javax.mail.internet.InternetAddress>) copiaEmailTo);
    	
    	if(isHtml)
    		this.email.setContent("<p>Ol�, <b>"+nomeRemetente+"</b> enviou a seguinte mensagem:</p> \n "+mensagem, "text/html");
    	else
    		this.email.setMsg("<p>Ol�, <b>"+nomeRemetente+"</b> enviou a seguinte mensagem:</p> \n "+mensagem); //conteudo do e-mail  
    	
    	this.email.setSentDate(new Date());
    	
    	//this.email.setAuthentication(this.authUser, this.authPass);  
    	
    	
    	
    	this.email.setSmtpPort(this.port);  
    	this.email.setSSL(true);  
    	this.email.setTLS(true);  
    
    	try {
    		this.email.send();
    		return "";
		
    	} catch (Exception e) {
			return e.getMessage();
		}
    }  
 
    /**
     * @param String pathArquivo, String descricao, String nome
     * @return void
     * @exception EmailException*/
    public void addAnexo(String pathArquivo, String descricao, String nome) throws EmailException{
    	this.anexo = new EmailAttachment();  
    	this.anexo.setPath(pathArquivo); //caminho do arquivo (RAIZ_PROJETO/teste/teste.txt)  
    	this.anexo.setDisposition(EmailAttachment.ATTACHMENT);  
    	this.anexo.setDescription(descricao);  
    	this.anexo.setName(nome);          
        this.email.attach(anexo); 
    }
   
}  